@make(ais)
@aiInfo(topic=<Instances having implicit package bodies>,
number=[AI-00237/06],
classcode=[BI],
statuscode=[WJ],
mainref=[12.03@\(17)],
date=[86-07-23])
@aiHeading()
@begin(history)
@begin(toplines)
@mainref()
@standard[09.03@\(05)]
!class binding interpretation 84-04-13
!status approved by WG9/AJPO 86-07-22
@end(toplines)
!status approved by Director, AJPO 86-07-22
!status approved by WG9/Ada Board 86-07-22
!status approved by Ada Board 86-07-22
!status approved by WG9 86-05-09
!status committee-approved (8-0-0) 86-02-21
!status work-item 85-02-04
!status received 84-04-13
@references[!references @CommentNum[83-00348]83-00348, @CommentNum[83-00591]83-00591, @CommentNum[83-00708]83-00708]
@topic{!topic @value(aitopic)}
@end(history)

@subheading[!summary 86-04-02]

Given a generic package declaration that does not require a body and that
has no explicit body, when the generic package is instantiated, if the
instance specification requires a body, then an implicit instance body is
created and is elaborated when the instantiation is elaborated.

@subheading[!question 86-03-11]

Consider the following sequence of compilations:
@begin(example)
@key[with] TASK_@|TYPES;
@key[generic]
@key[package] SEMAPHORE_@|TASKS @key[is]
     S1, S2 : TASK_@|TYPES.SEMAPHORE_@|TYPE;  -- declare two task objects
@key[end] SEMAPHORE_@|TASKS;
@hinge()

@key[with] SEMAPHORE_@|TASKS;
@key[procedure] MAIN @key[is]
     I : INTEGER;
     @key[package] SEM @key[is] @key[new] SEMAPHORE_@|TASKS;
     -- (1)
     J : INTEGER;
     -- (2)
@key[begin]
     @key[null];
@key[end] MAIN;
@end(example)
Note that SEMAPHORE_@|TASKS is not required to have a body.  If the user
doesn't supply a body for SEMAPHORE_@|TASKS, then the compiler must create an
implicit one for SEM, in order to activate the tasks SEM.S1 and SEM.S2.
Where is this implicit body elaborated, i.e., are SEM.S1 and SEM.S2 activated
at point (1) (as implied by 12.3(17)) or at point (2) (as implied by 9.3(5))?

The same problem also occurs when task types are passed as actual generic
parameters to a generic package that otherwise has no tasks, as in:
@begin(example)
@key[generic]
     @key[type] LP @key[is] @key[limited] @key[private];
@key[package] P @key[is]
     OBJ : LP;
@key[end] P;
@hinge()

@key[with] P, TASK_@|TYPES;
@key[package] Q @key[is]
     @key[package] NP @key[is] @key[new] P (TASK_@|TYPES.SEMAPHORE_@|TYPE);
     -- (1)
     K : INTEGER;
     -- (2)
@key[end] Q;
@end(example)
Again, an implicit body is needed to activate the task NP.OBJ, but is this
body elaborated at point (1) or at point(2)?

@subheading[!recommendation 86-04-02]

Given a generic package declaration that does not require a body and that
has no explicit body, when the generic package is instantiated, if the
instance specification requires a body, then an implicit instance body is
created and is elaborated when the instantiation is elaborated.

@subheading[!discussion 86-03-11]

9.3(5) says:
@begin(quotation)
For any package  without a  package body,  an implicit  package
body containing  a  single null  statement  is assumed.   If  a
package without a package  body is declared immediately  within
some program unit or block statement, the implicit package body
occurs at the end of the declarative part@ellipsis()
@end(quotation)
12.3(17) says:
@begin(quotation)
For the elaboration of a generic instantiation@ellipsis()the implicitly
generated instance is elaborated.
@end(quotation)
From 9.3(5), it is clearly intended that package instances should have
implicit bodies so the task objects can be activated.  However, the
phrase @dq{the implicitly generated instance} in 12.3(17) is intended to refer
to both the instance specification and the instance body, i.e., any body
associated with an instance is to be elaborated when the instantiation is
elaborated.  In particular, the intent is that any implicit instance bodies
be elaborated at the points of instantiation (points (1)), and hence, task
objects declared within the instances are to be activated at points (1).

